function femmodel=thermal_core(femmodel)
%THERMAL_CORE - core of thermal solution
%
%   Usage:
%      femmodel=thermal_core(femmodel)


	%recover parameters common to all solutions
	verbose=femmodel.parameters.Verbose;
	ndt=femmodel.parameters.Ndt;
	dt=femmodel.parameters.Dt;
	control_analysis=femmodel.parameters.ControlAnalysis;
	solution_type=femmodel.parameters.SolutionType;

	%Compute number of timesteps
	if (dt==0 | ndt==0),
		dt=0;
		nsteps=1;
	else
		nsteps=floor(ndt/dt);
	end

	%Loop through time
	for i=1:nsteps,
		issmprintf(verbose,'\n%s%i/%i\n','time step: ',i,nsteps);
		time=(i+1)*dt;

		issmprintf(verbose,'\n%s',['computing temperature...']);
		femmodel=thermal_core_step(femmodel,i,time); 

		if (solution_type==ThermalSolutionEnum & ~control_analysis),
			issmprintf(verbose,'\n%s',['saving results...']);
			femmodel.elements=InputToResult(femmodel.elements,femmodel.nodes,femmodel.vertices,femmodel.loads,femmodel.materials,femmodel.parameters,TemperatureEnum,i,time);
			femmodel.elements=InputToResult(femmodel.elements,femmodel.nodes,femmodel.vertices,femmodel.loads,femmodel.materials,femmodel.parameters,MeltingRateEnum,i,time);
		end
	end

end %end of function
